#!/usr/bin/perl

# search.cgi

# This is a cgi that will search the registry based on certain 
# inputs from the users.
#
# Copyright (C) 2002
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Brian Rassier, University of Minnesota, Duluth
# rass0028@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#-----------------------------------------------------------------------------
#                              Start of program
#-----------------------------------------------------------------------------
use CGI;

#make a new cgi object to use throughout the script
$cgi = new CGI();

#print a header
print $cgi->header();

#print the top portion of the root html page
print <<EndHTML;
<html><head><title>Search Registry of Latin American Researchers</title></head>
    <body BGCOLOR = "ffffff">
    <h2 align ="CENTER">Search Registry of Latin American Researchers</h2>
<font color = ff0000><p align = center> Note: To update your entry, search for your name and press the "update entry" button next to your information.</p></font> 
EndHTML

    #used for seeing what types of input there are
    #search by letter, or by different fields.
    $inputflag = 0;

#finds the query string if a user selects a 
#letter link to find names beginning with that letter
if($ENV{'QUERY_STRING'}){

    #split the '&' in the query string
    @values = split(/&/,$ENV{'QUERY_STRING'});

    #split the = to find the letter
    foreach $i (@values){
	($varname, $mydata) = split(/=/,$i);
	$in{$varname} = $mydata;
    }
}

else{
    #if no querystring, undefine the querystring hash
    undef(%in);
}

#save the letter/member that was pressed
$search = $in{'az'};
$member = $in{'member'};

#clear out old alpha hash, which finds the number of names
#that begin with each letter.
%alpha = ();
    
#open the registry to get the data in
open(INFP,"approvedadd/registry.txt") or dienice("Couldn't open registry.txt: $! \n");

#lock the file in case of multiple users
flock(INFP,2);

#move to the beginning of the file
seek(INFP,0,0);

#store the data in @data
@data = <INFP>;

#close file
close(INFP);

#if data is found
if(@data){
    #go through each line of data
    foreach $i (@data){

	#remove newline
	chomp($i);

	#split the information in $i
	($num,$status, $last_name, $first_name, $affiliation, $department, $address, $address2, $city, $state, $postal_code,$country, $homepage, $homepage2, $email, $email2, $research, $origin_country,$date) = split(/\|/, $i);

	#store the file based on last name into the HashOfArrays.
	#HashOfArrays is a hash of the people based on last name.
	#This takes care of the issue of two people having the same last name.

	#if the hash of arrays already has the last name in it, add another array 
	#to the hash of that last name
	if($HashOfArrays{$last_name}){
	    my @temp = @{$HashOfArrays{$last_name}};
	    push @temp,$i;
	    $HashOfArrays{$last_name} = \@temp;
	}

	#else, add a new array for the hash of that last name.
	else{
	    my @temp = ();
	    push @temp, $i;
	    $HashOfArrays{$last_name} = \@temp;
	}

	#make a hash based on member numbers
	$HashOfNums{$num} = $i;

	#remove underscores from the values
	$country =~ s/_/ /g;
	$origin_country =~ s/_/ /g;
	
    }
}

#find the number of names that start with each letter
find_namelinks();

#calls the search function if a keyword was given (default)
if($cgi->param('search_keywords')){
    search(4,$cgi->param('search_keywords'));

    #increment inputflag noting that a parameter was true coming in to the cgi
    $inputflag++;
}

#calls the search function if a country of origin was given 
elsif($cgi->param('search_ocountry') && $cgi->param('search_ocountry') ne "none" ){
    search(3,$cgi->param('search_ocountry'));

    #increment inputflag noting that a parameter was true coming in to the cgi
    $inputflag++;
}

#calls the search function if a country of residence was given 
elsif($cgi->param('search_country') && $cgi->param('search_country') ne "none"){
    search(2,$cgi->param('search_country'));

    #increment inputflag noting that a parameter was true coming in to the cgi
    $inputflag++;
}


#does the letter search if no other input was given besides the letter link that was clicked
if(($search || $member) && ($inputflag == 0)){
    #if a letter was pressed, print names with that letter
    if($in{'search'} eq "az"){
	print_az($search);
    }
    #if full entry was pressed, find the entry and print it
    elsif($in{'search'} eq "print"){
	findentry($in{'member'});
    }
}

#search was pressed from main cgi, print the search fields
elsif(!($search || $member) && ($inputflag == 0)){

    #prints the search fields
    print_top();
}

#prints the tags for the end of the page
print "</body></html>";  

#this function prints the top of the search page, with the search options and fields
sub print_top{
    #Clear the hashes of used countries
    undef %h1;
    undef %h2;

#prints the intro portion of the web page
    print <<EndHTML;
<form method = post>
    <p> <b>Search Registry</b><i> (Please search one field only, or use the keywords field to search the total entry.)</i></p>
EndHTML

#Sets up a Latin American hash table
#This is used to tell if a country is from Latin America or not.
$latin{'Antigua'}++;
    $latin{'Argentina'}++;
    $latin{'Bahamas'}++;	                                               
    $latin{'Barbados'}++;						
    $latin{'Belize'}++;							
    $latin{'Bolivia'}++;						
    $latin{'Brazil'}++;						
    $latin{'Chile'}++;						
    $latin{'Colombia'}++;						
    $latin{'Costa_Rica'}++;						
    $latin{'Cuba'}++;						
    $latin{'Dominica'}++;						
    $latin{'Dominican_Republic'}++;						
    $latin{'Ecuador'}++;						
    $latin{'El_Salvador'}++;						
    $latin{'Grenada'}++;					
    $latin{'Guatemala'}++;						
    $latin{'Guyana'}++;						
    $latin{'Haiti'}++;						
    $latin{'Honduras'}++;						
    $latin{'Jamaica'}++;						
    $latin{'Mexico'}++;						
    $latin{'Nicaragua'}++;						
    $latin{'Panama'}++;						
    $latin{'Paraguay'}++;						
    $latin{'Peru'}++;						
    $latin{'Saint_Kitts_&_Nevis'}++;						
    $latin{'Saint_Lucia'}++;                                                 
    $latin{'Saint_Vincent_&_Grenadines'}++;		
    $latin{'Suriname'}++;						
    $latin{'Trinidad_&_Tobago'}++;						
    $latin{'Uruguay'}++;						
    $latin{'Venezuela'}++;						

    #This group of statements prints out the search fields including keyword,
    #latin residence country, latin original country, world residence country,
    #and world original country
    print "<table><tr><td><pre>Keywords:</td><td><input type = \"text\" name = \"search_keywords\" ></td><pre><td> <select name=\"search_country\">";
    print "<option value=\"none\">Country of Residence</option>";

    #This foreach nested loop stores what countries of Residence are in the registry
    #for residence countries in %h1
    foreach $name (sort keys %HashOfArrays){
	foreach (@{$HashOfArrays{$name}}) {
	    #split each entry
	    ($num,$status, $last_name, $first_name, $affiliation, $department, $address, $address2, $city, $state, $postal_code,$country, $homepage, $homepage2, $email, $email2, $research, $origin_country,$date,$location) = split(/\|/, $_);
	    #increment the country in hash %h1
	    $h1{$country}++;
	}
    }

    #print option group for latin america
    print "<optgroup label=\"----Latin America----\">";

    #goes through the latin country hash, and adds them to the select box
    #if they are in the registry (aka in %h1)
    foreach $ctry (sort keys %latin){
	my $tempcountry = $ctry;
	
	#remove underscores for better viewing
	$tempcountry =~ s/ /_/g;
	$ctry =~ s/_/ /g;

	#print the value of the country if they are in the registry (%h1)
	if($h1{$ctry}){print "<option VALUE=$tempcountry>$ctry";}
    }

    #print option group for latin america
    print "<optgroup label=\"----Rest of World----\">";

    #goes through the countries that are in the registry, and adds them
    #to the select box for searching for the world, unless they are latin countries
    foreach $ctry (sort keys %h1){
	my $tempcountry = $ctry;

	#remove underscores for better viewing
	$tempcountry =~ s/ /_/g;

	#print the value of the country if they are in the registry (%h1), unless they are latin countries
	print "<option VALUE=$tempcountry>$ctry" unless($latin{$tempcountry});
    }


    #start the new select box
    print " </select></td><pre><td><select name = \"search_ocountry\">";
    print "<option value=\"none\">Country of Origin</option>";
   
    #This foreach nested loop stores what Origin countries are in the registry
    #for residence countries in %h2
    foreach $name (sort keys %HashOfArrays){
	foreach (@{$HashOfArrays{$name}}) {
	    #split each entry
	    ($num,$status, $last_name, $first_name, $affiliation, $department, $address, $address2, $city, $state, $postal_code,$country, $homepage, $homepage2, $email, $email2, $research, $origin_country,$date,$location) = split(/\|/, $_);

	    #increment that country in %h2
	    $h2{$origin_country}++;
	}
    }

    #print option group for latin america
    print "<optgroup label=\"----Latin America----\">";

    #goes through the latin country hash, and adds them to the select box
    #if they are in the Origin Country portion of the registry (aka in %h2)
    foreach $octry (sort keys %latin){
	my $tempcountry = $octry;

	#remove underscores for better viewing
	$tempcountry =~ s/ /_/g;
	$octry =~ s/_/ /g;

	#print the value of the origin country if they are in the registry (%h2)
	if($h2{$octry}) {print "<option VALUE=$tempcountry>$octry";}
    }

    #print option group for latin america
    print "<optgroup label=\"----Rest of World----\">";    


    #goes through the countries that are in the registry, and adds them
    #to the select box for searching for the world, unless they are latin countries
    foreach $octry (sort keys %h2){
	my $tempcountry = $octry;

	#remove underscores
	$tempcountry =~ s/ /_/g;

	#print the option unless they are latin countries (which have already been printed)
	print "<option VALUE=$tempcountry>$octry" unless($latin{$tempcountry});
    }

#print all the ending links and tags for the page
print "</select></td></tr></table><br><table><tr><td><input type = \"submit\" name = \"submit_search\" value = \"Submit Search\"></td></form>";
print "<form method=post action = \"registry.cgi\">";
print "<td><input type = \"submit\" name = \"back\" value = \"Done Searching\"></td></tr></table></form><hr noshade></pre>";

print "Browse by Last Names:<br>";

#call the function that prints the links for each letter
print_namelinks();

}


#this function prints the smaller version of the entries
sub print_small{
    my $entry = shift;
    #split the information
    ($num,$status, $last_name, $first_name, $affiliation, $department, $address, $address2, $city, $state, $postal_code,$country, $homepage, $homepage2, $email, $email2, $research, $origin_country,$date,$location) = split(/\|/, $entry);

    #correct the homepage syntax (adds http:// if necessary)
    $link = $homepage;
    $link="http://".$link  unless $link =~ /\/\/|none|empty/gi;

    #this print statement prints the homepage information of the given user 
    if($homepage eq "none" || $homepage eq "empty"|| $homepage eq ""){
	print "<tr><td>$last_name, $first_name</td>";
    }
    else{
	print "<tr><td><a href=$link>$last_name, $first_name</a></td>";
    }
    print "<td>$country</td>";
    
    #change the spaces to underscores
    $last_name =~ s/ /_/g;
    $first_name =~ s/ /_/g;

    #print links at the bottom of the page
    print qq'
        <td>[<a href= "search.cgi?search=print&member=$num">Full Entry</a>] 
    	[<a href= "update.cgi?member=$num">Update Entry</a>] </td></tr>
';
}

#this function finds the entry that matches last name, first name, and email, and prints the full entry
sub findentry{
    #get member number that was passed in
    my $mem = shift;

    #print the member
    print_visitor($HashOfNums{$mem});

    #print a "back" link at the bottom of the page
    print "<br><a href = search.cgi>Back to Search</a>";

}


#This function is used to print out a visitor's information if a search gets results
sub print_visitor{
    my $entry = shift;
    #split the information
    ($num,$status, $last_name, $first_name, $affiliation, $department, $address, $address2, $city, $state, $postal_code,$country, $homepage, $homepage2, $email, $email2, $research, $origin_country,$date,$location) = split(/\|/, $entry);

    #correct the homepage syntax (adds http:// if necessary)
    if($homepage ne ""){
	$link = $homepage;
	$link="http://".$link  unless $link =~ /\/\/|none|empty/gi;
    }
    if($homepage2 ne ""){
	$link2 = $homepage2;
	$link2="http://".$link2  unless $link2 =~ /\/\/|none|empty/gi;
    }


    #this large print statement prints the information of the given user 
    print <<EndHTML;
    <table border = 10 noshade>
      <tr bgcolor = "ffff00"><td>
      Name:
	</td><td>$last_name, $first_name
	</td></tr>
	<tr><td>
      Job title/position::
	</td><td>
EndHTML
if($status =~ /^Student/){
    ($temp1, $temp2, $temp3) = split (/%%%/,$status);
     print "Student</td></tr><tr><td>Student Advisor/Supervisor:</td><td>$temp2</td></tr><tr><td>";
     print "Advisor/Supervisor's Email:</td><td><a href = mailto:$temp3>$temp3</a></td></tr>";
 }
     else{
	 $status =~ s/%%%/:/g;
	 print "$status </td></tr>";
     }
     print <<EndHTML;
	<tr><td>
	Organization:  
	</td><td>$affiliation
	</td></tr>
	<tr><td>
	Department:  
	</td><td>$department
	</td></tr>
	<tr><td>
	Address line 1:
	</td><td>$address
	</td></tr>
	<tr><td>
	Address line 2:
	</td><td>
EndHTML
if($address2 eq ""){
    print "none";
}
    else{
	print "$address2";
    }
    print <<EndHTML;
	</td></tr>
	<tr><td>
	City:
	</td><td>$city
	</td></tr>
	<tr><td>
	State/Province:
	</td><td>$state
	</td></tr>
	<tr><td>
        Postal Code:
	</td><td>$postal_code
	</td></tr>
	<tr><td>
	Country of Residence: 
	</td><td>$country
	</td></tr>
	<tr><td>
EndHTML
	
#prints the homepage and email as a link, unless "none" or "empty" were given
if($homepage eq ""){
    print "Personal Homepage: </td><td>none</td></tr><tr><td>";
}
    else{
	print "Personal Homepage:</td><td>  <a href=$link>$link</a></td></tr><tr><td>";
    }

    if($homepage2 eq ""){
	print "Departmental/Group Homepage: </td><td>none</td></tr><tr><td>";
    }
    else{
	print "Departmental/Group Homepage: </td><td> <a href=$link2>$link2</a></td></tr><tr><td>";
    }

    if($email eq "none" || $email eq "empty"){
	print "Email: </td><td>none</td></tr><tr><td>";
    }
    else{
	print "Email:</td><td>  <a href = mailto:$email>$email</a></td></tr><tr><td>";
    }

    if($email2 eq "none" || $email2 eq "empty" || $email2 eq ""){
	print "Alternate Email: </td><td>none</td></tr><tr><td>";
    }
    else{
	print "Alternate Email:</td><td>  <a href = mailto:$email2>$email2</a></td></tr><tr><td>";
    }
    
    #change spaces to underscores
    $last_name =~ s/ /_/g;
    $first_name =~ s/ /_/g;

    #prints the end of the data (research interests and origin country)
    print <<EndHTML;
    Research Interests:
    </td><td> $research
	</td></tr>
	<tr><td>
	Country of Origin:
	</td><td>$origin_country</td></tr></table>
	<i>Last updated: $date</i><br>
	<a href= "update.cgi?member=$num">Update Entry</a><br><br>
	
EndHTML
}

#This function finds the number of last names that start with each letter
sub find_namelinks{

    #for each entry in the registry...
    foreach $name (sort keys %HashOfArrays){
	foreach (@{$HashOfArrays{$name}}) {
	
	#gets the last name into $temp
	my $temp = $name;
	
	#gets the first letter of the last name
	$temp =~ s/^([A-Za-z]).*/$1/g;
	
	#makes the letter upper-case so repeats don't happen with upper/lower case
	$temp = uc($temp);
	
	#increments the hash with that letter 
	$alpha{$temp}++;
    }
    }
}

#This function prints the links for the last names that start with each letter
sub print_namelinks{
    print "<pre>";
    
    #for each letter
    foreach ("A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z"){
	#if there are no last names with this letter, make it 0 
	$alpha{$_} = 0 unless($alpha{$_});

	#print the link, unless there are no names that start with this letter
	print "<a href=\"search.cgi?search=az&az=$_\">$_ </a>($alpha{$_})" unless ($alpha{$_} == 0);
    }
    print "</pre>";
}

#This function prints all the last names that begin with a certain letter
sub print_az{
    #get the letter that was passed in
    my $search = shift;
    
    #keeps track of how many names were printed
    my $printcount = 0;
    
    #prints the top of the webpage with the search fields
    print_top();

    #goes through each name in the database
    foreach $name (sort keys %HashOfArrays){
	#gets the last name
	my $temp = $name;
	
	#finds the first letter of the last name
	$temp =~ s/^([A-Za-z]).*/$1/g;
	
	#makes the letter uppercase if it is not originally
	$temp = uc($temp);

	#print the person's information if the letters match
	if($temp eq $search){
	    #for each element in the array of this hash
	    foreach (@{$HashOfArrays{$name}}) {
		if($printcount == 0){
		    print "<table border = 2><tr bgcolor = \"ffff00\"><td>Name</td><td>Country of Residence</td><td>Options</td></tr>";
		}
		print_small($_);
		#increment the print counter
		$printcount++;
	    }
	}
    }

    #prints "no hits" if there were no people found with that letter starting their last name
    if(!$printcount){
	print "<br>no hits";
    }
    else{
	print "</table>";
    }

}

#This function prints all the visitors with a certain word, depending on what they searched for
sub search{
    #finds the type of search that was passed in
    my $type = shift;

    #finds the word(s) to look for in the database
    my $field = shift;

    #removes underscores
    $field =~ s/_/ /gi;
    
    #keeps track of the number of people printed
    my $printcount = 0;
    
    #prints the top of the webpage with the search fields
    print_top();

    #search the country of residence depending on the type of search
    if($type == 2){
	#go through the hash of arrays looking for a matched country of residence
	foreach $name (sort keys %HashOfArrays){
	    foreach (@{$HashOfArrays{$name}}) {

		#split the information
		($num,$status, $last_name, $first_name, $affiliation, $department, $address, $address2, $city, $state, $postal_code,$country, $homepage, $homepage2, $email, $email2, $research, $origin_country,$date,$location) = split(/\|/, $_);
		
		#if a matcing country was found
		if($field =~ $country){

		    #if this is the first thing printed, start the table
		    if($printcount == 0){
			print "<table border = 2><tr bgcolor = \"ffff00\"><td>Name</td><td>Country of Residence</td><td>Options</td></tr>";
		    }
		    
		    #print the entry's small version and increment the counter
		    print_small($_);
		    $printcount++;
		}
		    
	    }
	}
    }

    
    #search the country of origin depending on the type of search
    if($type == 3){
	#go through the hash of arrays looking for a matched country of origin
	foreach $name (sort keys %HashOfArrays){
	    foreach (@{$HashOfArrays{$name}}) {
		#split the information
		($num,$status, $last_name, $first_name, $affiliation, $department, $address, $address2, $city, $state, $postal_code,$country, $homepage, $homepage2, $email, $email2, $research, $origin_country,$date,$location) = split(/\|/, $_);
		
		#if a matcing country was found
		if($field =~ $origin_country){
	
		    #if this is the first thing printed, start the table
		    if($printcount == 0){
			print "<table border = 2><tr bgcolor = \"ffff00\"><td>Name</td><td>Country of Residence</td><td>Options</td></tr>";
		    } 

		    #print the entry's small version and increment the counter
		    print_small($_);
		    $printcount++;
		}
		
	    }
	}
    }
 

    #search for keywords (the rest of the fields) depending on the type of search
    if($type == 4){
	#for each entry in the registry
	foreach $name (sort keys %HashOfArrays){
	    foreach(@{$HashOfArrays{$name}}){
		($num,$status, $last_name, $first_name, $affiliation, $department, $address, $address2, $city, $state, $postal_code,$country, $homepage, $homepage2, $email, $email2, $research, $origin_country) = split(/\|/, $_);
		
		#match if any keyword has the inputted field in it, for all fields
		if($status =~ /\b$field\b/gi || $last_name =~ /\b$field\b/gi || $first_name =~ /\b$field\b/gi || $affiliation =~ /\b$field\b/gi || $department =~ /\b$field\b/gi || $address =~ /\b$field\b/gi || $address2 =~ /\b$field\b/gi || $city =~ /\b$field\b/gi || $state =~ /\b$field\b/gi || $postal_code =~ /\b$field\b/gi || $homepage =~ /\b$field\b/gi || $homepage2 =~ /\b$field\b/gi || $email =~ /\b$field\b/gi || $email2 =~ /\b$field\b/gi || $research =~ /\b$field\b/gi || $field =~ /$country/gi ||  $field =~ /$origin_country/gi ){
		#prints the record
		if($printcount == 0){
		    print "<table border = 2><tr bgcolor = \"ffff00\"><td>Name</td><td>Country of Residence</td><td>Options</td></tr>";
		}
		print_small($_); 
		$printcount++;
	    }
	    }
	}
    }

    #print "no hits" if there were no visitors printed out
    if($printcount == 0){
	print "<br> no hits";
    }
    else{
	print "</table>";
    }
    
}


#function will print an error message
sub dienice{
    my($msg) = @_;
    print "<h2Error</h2>\n";
    print $msg;
    exit;
}
