#!/usr/bin/perl

# approve.cgi
#
# This is a cgi that will allow a administrator to approve
# or reject additions/updates to the registry.
#
# Copyright (C) 2002
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Brian Rassier, University of Minnesota, Duluth
# rass0028@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#-----------------------------------------------------------------------------
#                              Start of program
#-----------------------------------------------------------------------------

use CGI;

#cgi is the cgi object that is passed back and forth, reply
#is the reply-to address on the rejection messages
use vars qw($CGI $REPLY );

$CGI = new CGI();

#This is the address which will be used for the 
#reply section of automated emails.
$REPLY = "tpederse\@umn.edu";

print $CGI->header();


#THIS BLOCK OF IF STATEMENTS CONTROLS THE FLOW OF
#THE WHOLE PROGRAM.
#goes to the approve additions feature
if($CGI->param("submit_additions")){
    checkadds($CGI);
}
#will show all the additions to approve, but
#will not give the ability to approve/reject here.
elsif($CGI->param("submit_inspectadds")){    
    viewadds($CGI);
}
#goes to the approve updates feature
elsif($CGI->param("submit_updates")){    
    checkupdates($CGI);
}
#will show all the updates to approve, but
#will not give the ability to approve/reject here.
elsif($CGI->param("submit_inspectupdates")){    
    viewupdates($CGI);
}
#adds the entry, then moves on to the next entry
elsif($CGI->param("approve_add")){    
    approveadd();
    checkadds($CGI);
}
#deletes the entry, then allows a message to be sent
#to the user who entered the entry
elsif($CGI->param("reject_add")){    
    rejectadd($CGI);
}
#updates the entry, then moves on to the next entry
elsif($CGI->param("approve_update")){    
    approveupdate();
    checkupdates($CGI);
}
#deletes the update, then allows a message to be sent
#to the user who entered the entry
elsif($CGI->param("reject_update")){    
    rejectupdate($CGI);
}
#Sends the rejection reasons to the person who was rejected
elsif($CGI->param("send_rejection")){    
    sendrejection($CGI);
}
#rejects adds without sending a message
elsif($CGI->param("nosend_rejection")){    
    checkadds($CGI);
}
#rejects updates without sending a message
elsif($CGI->param("nosendupdate_rejection")){    
    checkupdates($CGI);
}
#If no other inputs were given, go to the default
#root page.
else{
    rootapproval($CGI);
}

sub rootapproval{
#this print statement will print out the whole html page for the root approval page.
    print qq'
<html>
  <head>
    <title>Entry Approval</title>
   
</head>

<body bgcolor="#ffffff">

  <h2 align = "center">Entry Approval</h2>

<p align = "center">Approve first-time additions, or updates</p><br>

<hr size = 5 noshade>
<form method = "POST">
<table>
   <tr><td><input type="submit" name = "submit_additions" value="Approve Additions"></td>
   <td> <input type="submit" name = "submit_updates" value="Approve Updates"></td></tr>
   <tr><td><input type="submit" name = "submit_inspectadds" value="View All Additions"></td>
   <td> <input type="submit" name = "submit_inspectupdates" value="View All Updates"></td></tr></form></table><hr size=5 noshade>
<table><tr><td>
<form method = "POST" action= "delete.cgi">
<input type="submit" name = "delete_entry" value="Delete a Registry Entry"></form></td><td>
<form method = "POST" action= "undelete.cgi">
<input type="submit" name = "undelete_entry" value="Re-add a Deleted Entry"></form>
</td></tr></table>

</body>
</html>

';
}


#this function will show the additions, and allow someone to approve them
#it will print out its own html page
sub checkadds{

    #get the cgi that was passed in
    my $cgi = shift;
    
    #print $cgi->header();

    #this print statement will print out the top of the html page.
    print qq'
<html>
  <head>
    <title>First-time addition approval</title>
   
</head>

<body bgcolor="#ffffff">

  <h2 align = "center">First-time addition Approval</h2>

<hr noshade>
    
';

#Open pendingadd.txt to get the next addition to approve/reject
open(INFP,"../pendingadd/pendingadd.txt") or dienice("Couldn't open pendingadd.txt: $! \n");

#lock the file in case of multiple users
flock(INFP,2);

#move to the beginning of the file
seek(INFP,0,0);

#store the next entry to be approved in $data
$data = <INFP>;

#close file
close(INFP);

#if data is found (there have been entries)
if($data){

	#remove newline
	chomp($data);

	#split the information apart into seperate fields
	($num,$status, $last_name, $first_name, $affiliation, $department, $address, $address2, $city, $state, $postal_code,$country, $homepage, $homepage2, $email, $email2, $research, $origin_country,$date,$location) = split(/\|/, $data);
	
	#remove underscores from the values
	$country =~ s/_/ /g;
	$origin_country =~ s/_/ /g;
	
	#entries are marked with a A: for past approval or R: for past
	#rejection.  This checks for the past approval/rejection, meaning
	#if the entry has either, it has already been processed, and there 
	#are no new entries.
	if($num =~ /^R:.+/ || $num =~ /^A:.+/ || $num eq ""){
	    print "<b>No additions to approve</b>";
	    print "<br><br><a href = approve.cgi>Back</a>";
	    
	}
	#else there are entries that have not been processed
	else{

	    #print the entry for aproval
	    print_visitor();

	    #print the html form submission information
	    print qq'
        <br>
	<form method = "post">
	   <input type="submit" name = "approve_add" value="Approve">
   <input type="submit" name = "reject_add" value="Reject">
</form>
<a href = approve.cgi>Back</a>
</body>
</html>
	
';
	}
    }
    
}



#this function will show the updates, and allow someone to approve them
#it will print out its own html page
sub checkupdates{

    #get the cgi that was passed in
    my $cgi = shift;
    
    #print $cgi->header();

    #this print statement will print out the start of the html page.
    print qq'
<html>
  <head>
    <title>Entry update approval</title>
   
</head>

<body bgcolor="#ffffff">

  <h2 align = "center">Entry Update Approval</h2>

<hr noshade>
    
';

    #open pendingupdate.txt to get the next update to be processed
    open(INFP,"../pendingupdate/pendingupdate.txt") or dienice("Couldn't open pendingupdate.txt: $! \n");
    
    #lock the file in case of multiple users
    flock(INFP,2);

    #move to the beginning of the file
    seek(INFP,0,0);

    #store the first two lines, which makes one update
    #$data1 = original entry; $data2 = updated entry
    $data1 = <INFP>;
    $data2 = <INFP>;

    #close file
    close(INFP);

    #if data is found
    if($data1 && $data2){
    
	#remove newline
	chomp($data1);
	chomp($data2);
	
	#split the original information
	($num,$status, $last_name, $first_name, $affiliation, $department, $address, $address2, $city, $state, $postal_code,$country, $homepage, $homepage2, $email, $email2, $research, $origin_country,$date,$location) = split(/\|/, $data1);
	#split the updated information
	($n,$st, $la, $fi, $af, $de, $ad, $ad2, $ci, $ste, $po,$co, $ho, $ho2, $em, $em2, $re, $or) = split(/\|/, $data2);
	    
	#keep a hash telling which fields have changed.  
	#the %diff hash stores this information for printing
	#out updates.  This way the maintainer can tell which
	#fields have changed, to process the entry easier.
	$diff{'status'}++ if($status ne $st);
	$diff{'first_name'}++ if($first_name ne $fi);
	$diff{'last_name'}++ if($last_name ne $la);
	$diff{'affiliation'}++ if($affiliation ne $af);
	$diff{'department'}++ if($department ne $de);
	$diff{'address'}++ if($address ne $ad);
	$diff{'address2'}++ if($address2 ne $ad2);
	$diff{'city'}++ if($city ne $ci);
	$diff{'state'}++ if($state ne $ste);
	$diff{'postal_code'}++ if($postal_code ne $po);
	$diff{'country'}++ if($country ne $co);
	$diff{'homepage'}++ if($homepage ne $ho);
	$diff{'homepage2'}++ if($homepage2 ne $ho2);
	$diff{'email'}++ if($email ne $em);
	$diff{'email2'}++ if($email2 ne $em2);
	$diff{'research'}++ if($research ne $re);
	$diff{'origin_country'}++ if($origin_country ne $or);

	#remove underscores from the values for better printing
	$country =~ s/_/ /g;
	$origin_country =~ s/_/ /g;
	

	#entries are marked with a A: for past approval or R: for past
	#rejection.  This checks for the past approval/rejection, meaning
	#if the entry has either, it has already been processed, and there 
	#are no new entries.
	if($num =~ /^R:.+/ || $num =~ /^A:.+/){
	    print "<b>No updates to approve</b>";
	    print "<br><br><a href = approve.cgi>Back</a>";
	    
	}

	#else there are entries that have not been processed
	else{

	    #Start an html table for the entries
	    print "<table> <tr><td>Original Entry</td><td></td><td>New Entry</td></tr><tr><td>";
	    #call print_visitor with $data1.  Data1 has been split using
	    #the proper field names.
	    print_visitor();
	    print "</td><td></td><td>";

	    #split the information for $data2 so print_visitor can be used
	    #on $data2
	    ($num,$status, $last_name, $first_name, $affiliation, $department, $address, $address2, $city, $state, $postal_code,$country, $homepage, $homepage2, $email, $email2, $research, $origin_country,$date,$location) = split(/\|/, $data2);
	
	    #remove underscores from the values for better printing
	    $country =~ s/_/ /g;
	    $origin_country =~ s/_/ /g;
	    
	    #call print_visitor to print the updated entry ($data2)
	    print_visitor();
	    print "</td></tr></table>";

	    #print the html form submission information
	    print qq'
<br>
	<form method = "post">
	   <input type="submit" name = "approve_update" value="Approve">
   <input type="submit" name = "reject_update" value="Reject">
</form>
<a href = approve.cgi>Back</a>
</body>
</html>
	
';
 
	}
    }
    #else no data found, print this in html
    else{
	print "<b>No updates to approve</b>";
	print "<br><br><a href = approve.cgi>Back</a>";
    }
}


#This function is used to print out a visitor's information if a search gets results
#relys on diff hash for background color.  The diff hash is set for each field
#if an update has been done.  This shows which fields have been changed
#for easier approval to updates.
sub print_visitor{

    #correct the homepage syntax (adds http:// if necessary)
    if($homepage ne ""){
	$link = $homepage;
	$link="http://".$link  unless $link =~ /\/\/|none|empty/gi;
    }
    if($homepage2 ne ""){
	$link2 = $homepage2;
	$link2="http://".$link2  unless $link2 =~ /\/\/|none|empty/gi;
    }


    #this large block of print statement prints the information of the given 
    #user. It also checks %diff to see if each field differs, in the case 
    #of updates. If the fields changed, they are printed in red for easier
    #processing by the maintainer
    print "<table border = 10 noshade>";
    if($diff{'last_name'} || $diff{'first_name'}){
	print "<tr bgcolor = \"ff0000\"><td>";
    }
    else{
	print "<tr><td>";
    }
	
print <<EndHTML;			     
 	 <font size = 2>   Name:
    </td>
<td>
    <font size = 2>$last_name, $first_name
	</td></tr>
EndHTML


    if($diff{'status'}){
	if($status =~ /^Student/){
	    ($temp1, $temp2, $temp3) = split (/%%%/,$status);
	    print "<tr bgcolor = \"ff0000\"><td><font size = 2>Job title/position:</td><td><font size = 2>Student</td></tr><tr bgcolor = \"ff0000\"><td><font size = 2>Student Advisor/Supervisor:</td><td><font size = 2>$temp2</td></tr><tr bgcolor = \"ff0000\"><td>";
	    print "<font size = 2>Advisor/Supervisor's Email:</td><td><a href = mailto:$temp3><font size = 2>$temp3</a></td></tr>";
 }
	else{
	    $status =~ s/%%%/:/g;
	    print "<tr bgcolor = \"ff0000\"><td><font size = 2>Job title/position:</td><td><font size = 2>$status </td></tr>";
	}
    }
    else{
	if($status =~ /^Student/){
	    ($temp1, $temp2, $temp3) = split (/%%%/,$status);
	    print "<tr><td><font size = 2>Job title/position:</td><td><font size = 2>Student</td></tr><tr><td><font size = 2>Student Advisor/Supervisor:</td><td><font size = 2>$temp2</td></tr><tr><td>";
	    print "<font size = 2>Advisor/Supervisor's Email:</td><td><a href = mailto:$temp3><font size = 2>$temp3</a></td></tr>";
	}
	else{
	    $status =~ s/%%%/:/g;
	    print "<tr><td><font size = 2>Job title/position:</td><td><font size = 2>$status </td></tr>";
	}

    }



    if($diff{'affiliation'}){
	print "<tr bgcolor = \"ff0000\"><td>";
    }
    else{
	print "<tr><td>";
    }
    print <<EndHTML;
	<font size = 2>Organization:  
	</td><td><font size = 2>$affiliation
	</td></tr>
EndHTML
    if($diff{'department'}){
	print "<tr bgcolor = \"ff0000\"><td>";
    }
    else{
	print "<tr><td>";
    }
    print <<EndHTML;
	<font size = 2>Department:  
	</td><td><font size = 2>$department
	</td></tr>
EndHTML
    if($diff{'address'}){
	print "<tr bgcolor = \"ff0000\"><td>";
    }
    else{
	print "<tr><td>";
    }
    print <<EndHTML;
    <font size = 2>Address line 1:
    </td><td><font size = 2>$address
	    </td></tr>
EndHTML
    if($diff{'address2'}){
	print "<tr bgcolor = \"ff0000\"><td>";
    }
    else{
	print "<tr><td>";
    }
    print <<EndHTML;
    <font size = 2>Address line 2:
    </td><td><font size=2>$address2
	</td></tr>
EndHTML

    if($diff{'city'}){
	print "<tr bgcolor = \"ff0000\"><td>";
    }
    else{
	print "<tr><td>";
    }
    print <<EndHTML;
	<font size = 2>City:
	</td><td><font size = 2>$city
	</td></tr>
EndHTML
if($diff{'state'}){
    print "<tr bgcolor = \"ff0000\"><td>";
}
    else{
	print "<tr><td>";
    }
    print <<EndHTML;
    <font size = 2>State/Province:
    </td><td><font size = 2>$state
	</td></tr>
EndHTML
if($diff{'postal_code'}){
    print "<tr bgcolor = \"ff0000\"><td>";
}
    else{
	print "<tr><td>";
    }
    print <<EndHTML;
    <font size = 2>Postal Code:
    </td><td><font size = 2>$postal_code
	</td></tr>
EndHTML
if($diff{'country'}){
    print "<tr bgcolor = \"ff0000\"><td>";
}
    else{
	print "<tr><td>";
    }
    print <<EndHTML;
    <font size = 2>Country of Residence: 
    </td><td><font size = 2>$country
	</td></tr>
EndHTML
	
#prints the homepage and email as a link, unless none was given
if($homepage eq ""){
    if($diff{'homepage'}){
	print "<tr bgcolor = \"ff0000\"><td>";
    }
    else{
	print "<tr><td>";
    }
    print "<font size = 2>Personal Homepage: </td><td>none</td></tr>";
}
    else{
	if($diff{'homepage'}){
	    print "<tr bgcolor = \"ff0000\"><td>";
	}
	else{
	    print "<tr><td>";
	}
	print "<font size = 2>Personal Homepage:</td><td>  <a href=$link>$link</a></td></tr>";
    }


    if($homepage2 eq ""){
	if($diff{'homepage2'}){
	    print "<tr bgcolor = \"ff0000\"><td>";
	}
	else{
	    print "<tr><td>";
	}
	print "<font size = 2>Departmental/Group Homepage: </td><td>none</td></tr>";
    }
    else{
	if($diff{'homepage2'}){
	    print "<tr bgcolor = \"ff0000\"><td>";
	}
	else{
	    print "<tr><td>";
	}
	print "<font size = 2>Departmental/Group Homepage: </td><td> <a href=$link2>$link2</a></td></tr>";
    }
    
    	if($diff{'email'}){
	    print "<tr bgcolor = \"ff0000\"><td>";
	}
	else{
	    print "<tr><td>";
	}

	print "<font size = 2>Email:</td><td>  <a href = mailto:$email><font size = 2>$email</a></td></tr>";


    if($email2 eq "none" || $email2 eq "empty" || $email2 eq ""){
	if($diff{'email2'}){
	    print "<tr bgcolor = \"ff0000\"><td>";
	}
	else{
	    print "<tr><td>";
	}
	print "<font size = 2>Alternate Email: </td><td><font size = 2>none</td></tr>";
    }
    else{
	if($diff{'email2'}){
	    print "<tr bgcolor = \"ff0000\"><td>";
	}
	else{
	    print "<tr><td>";
	}
	print "<font size = 2>Alternate Email:</td><td>  <a href = mailto:$email2><font size = 2>$email2</a></td></tr>";
    }
    
    #prints the end of the data (research interests and origin country)
    if($diff{'research'}){
	print "<tr bgcolor = \"ff0000\"><td>";
    }
    else{
	print "<tr><td>";
    }
    print <<EndHTML;
    <font size = 2>Research Interests:
    </td><td> <font size = 2>$research
	</td></tr>
EndHTML
    if($diff{'origin_country'}){
	print "<tr bgcolor = \"ff0000\"><td>";
    }
    else{
	print "<tr><td>";
    }
    print <<EndHTML;
	<font size = 2>Country of Origin:
	</td><td><font size = 2>$origin_country</td></tr></table></font>
	    <i>updated at $date</i>
	    <br><i>Location entered from: $location</i><br> 
	
EndHTML
}

#this function approves an entry, and then moves on to the next entry.
sub approveadd{

    #open pendingadd.txt to get the entry to approve
    open(FP,"../pendingadd/pendingadd.txt") or dienice("Couldn't open pendingadd.txt: $! \n");

    #lock the file in case of multiple users
    flock(FP,2);

    #move to the beginning of the file
    seek(FP,0,0);

    #store the file in $data(next entry) and @rest(rest of the file)
    $data = <FP>;
    @rest = <FP>;

    #close file
    close(FP);

    #mark the entry with the approved sign (A:rest_of_entry)
    $data = "A:" . $data;

    #open file to change its contents
    open(FP,">../pendingadd/pendingadd.txt") or dienice("Couldn't open pendingadd.txt: $! \n");

    #lock the file in case of multiple users
    flock(FP,2);

    #print the rest of the file, if there is any info in it
    if(@rest){
	print FP @rest;
    }

    #print the entry at the end of the file
    print FP $data;
    
    #close file
    close (FP);

    #remove the A: sign so $data is in its original form
    $data =~ s/^A://;

    #open approvedadd.txt to make an entry noting that the entry was approved
    open(FP2,">>../approvedadd/approvedadd.txt") or dienice("Couldn't open approvedadd.txt: $! \n");

    #lock the file in case of multiple users
    flock(FP2,2);

    #move to the end of the file
    seek(FP2,0,2);

    #print the approved entry in the end of approvedadd.txt
    print FP2 "$data";

    #close approvedadd.txt 
    close(FP2);

    #open registry.txt to add the entry to the main registry file
    open(FP2,">>../approvedadd/registry.txt") or dienice("Couldn't open registry.txt: $! \n");

    #lock the file in case of multiple users
    flock(FP2,2);

    #move to the end of the file
    seek(FP2,0,2);

    #print the approved entry in the end of registry.txt
    print FP2 "$data";

    #close file
    close(FP2);

    #split the information in $data1(the entry) to get individual fields
    ($num,$status, $last_name, $first_name, $affiliation, $department, $address, $address2, $city, $state, $postal_code,$country, $homepage, $homepage2, $email, $email2, $research, $origin_country,$date,$location) = split(/\|/, $data);

    #make a variable holding the name of the transaction file for the entry
    $filename = $num."-".$date.".txt";
    
    #move transaction file to the approved directory
    if(!(rename "../pendingadd/$filename","../approvedadd/$filename")){
	dienice("Couldn't change $filename: $! \n");}

    #This section sends an automated email to the user who entered
    #the entry letting them know the entry was approved
$ENV{PATH} = "/bin:/usr/bin";
delete @ENV{ qw( IFS CDPATH ENV BASH_ENV ) };

#open the sendmail program
open MAIL, "| /usr/lib/sendmail -t -i" or dienice("Couldn't open sendmail: $! \n");


#############################################################
#CHANGE THE CONTENT OF THE AUTOMATED EMAIL HERE
#THIS IS THE AUTOMATED ADDITION APPROVAL MESSAGE
#############################################################
#Change the link here for the automated email for approvals.  
#To do this, use the $num variable so the entry that this link
#contains relates to the user's entry.  An example is given in
#the message which is: http://www.d.umn.edu/~rass0028/cgi-bin/post-test/browse.cgi?search=print&member=$num
#The portion of this link that must stay the same is /browse.cgi?search=print&member=$num
#The portion of the link that will change is the content before this section.
#It will change to where the maintainer has the site posted.  
print MAIL <<END_OF_MESSAGE;
To: $email
Reply-To: $REPLY
Subject: Registry of Latin American Researchers Approval
   

Your Addition to The Registry of Latin American Researchers in Natural 
Language Processing and Computational Linguistics has been approved.

Please check your entry for accuracy at:
http://www.d.umn.edu/~tpederse/registry/browse.cgi?search=print&member=$num 
and update as necessary.   

Make sure to come back and visit the Registry. We hope you'll find
the information useful. We'd be grateful for any comments or suggestions
you might have. 

Please tell your students, friends, colleagues, etc.  about the Registry.  
They can browse or join simply by visiting: 

http://www.d.umn.edu/~tpederse/registry/registry.cgi

Thank you for your participation!

END_OF_MESSAGE
#close sendmail
close MAIL or dienice("Couldn't close sendmail: $! \n");

}

#this function will approve an update to the registry
sub approveupdate{

    #open pendingupdate.txt to get the entry to approve
    open(FP,"../pendingupdate/pendingupdate.txt") or dienice("Couldn't open pendingupdate.txt: $! \n");

    #lock the file in case of multiple users
    flock(FP,2);

    #move to the beginning of the file
    seek(FP,0,0);

    #store the file in $data1, $data2 and @rest
    $data1 = <FP>;
    $data2 = <FP>;
    @rest = <FP>;

    #close file
    close(FP);

    #mark with the approve sign, stating that the entry has been processed
    $data1 = "A:" . $data1;
    $data2 = "A:" . $data2;

    #open pendingupdate.txt to print the data with the A: sign
    open(FP,">../pendingupdate/pendingupdate.txt") or dienice("Couldn't open pendingupdate.txt: $! \n");

    #lock the file in case of multiple users
    flock(FP,2);

    #change the file with the approved update marked, put the 
    #approved entry at the end of the file
    if(@rest){
	print FP @rest;
    }
    print FP $data1;
    print FP $data2;

    #close file
    close (FP);

    #remove the approval marking to use the entry in its original form
    $data1 =~ s/^A://;
    $data2 =~ s/^A://;

    #print the info to the approved file (approvedupdate.txt)
    open(FP2,">>../approvedupdate/approvedupdate.txt") or dienice("Couldn't open approvedupdate.txt: $! \n");

    #lock the file in case of multiple users
    flock(FP2,2);

    #move to the end of the file
    seek(FP2,0,2);

    #append the entry to the end of approvedupdate.txt
    print FP2 $data1;
    print FP2 $data2;

    #close file
    close(FP2);

    #open registry.txt to make the update to the registry itself
    open(FP2,"../approvedadd/registry.txt") or dienice("Couldn't open registry.txt: $! \n");

    #lock the file in case of multiple users
    flock(FP2,2);

    #move to the beginning of the file
    seek(FP2,0,0);

    #split the original information
    ($num,$status, $last_name, $first_name, $affiliation, $department, $address, $address2, $city, $state, $postal_code,$country, $homepage, $homepage2, $email, $email2, $research, $origin_country,$date,$location) = split(/\|/, $data1);

    #look for the entry to update, based on the member number.
    #save the file in @beginning and @end, stating that the update
    #should go in between.
    $temp = <FP2>;
    while(($temp !~ /^$num\|/)){
	push @beginning,$temp;
	$temp = <FP2>;
    }
    @end = <FP2>;

    #close file
    close(FP2);

    #open registry.txt with writing privilege to actually make the changes
    open(FP3,">../approvedadd/registry.txt") or dienice("Couldn't open registry.txt: $! \n");

    #lock the file in case of multiple users, seek to the beginning of the file
    flock(FP3,2);
    seek(FP3,0,0);

    #print the beginning of the file, the updated entry, then the end 
    print FP3 @beginning,$data2,@end;

    #close registry.txt
    close(FP3);

    
    #split the information for data2
    ($num,$status, $last_name, $first_name, $affiliation, $department, $address, $address2, $city, $state, $postal_code,$country, $homepage, $homepage2, $email, $email2, $research, $origin_country,$date,$location) = split(/\|/, $data2);

    #save the name for the transaction file in $filename
    $filename = $num."-".$date.".txt";

    #move transaction file to the approved directory
    if(!(rename "../pendingupdate/$filename","../approvedupdate/$filename")){
    dienice("Couldn't change $filename: $! \n");}

    #this section sends an automated email to the user who entered the 
    #entry so they know the entry was added.
$ENV{PATH} = "/bin:/usr/bin";
delete @ENV{ qw( IFS CDPATH ENV BASH_ENV ) };

#open the sendmail program
open MAIL, "| /usr/lib/sendmail -t -i" or dienice("Couldn't open sendmail: $! \n");

#############################################################
#CHANGE THE CONTENT OF THE AUTOMATED EMAIL HERE
#THIS IS THE AUTOMATED UPDATE APPROVAL MESSAGE
#############################################################
#send message 
#Change the link here for the automated email for approvals.  
#To do this, use the $num variable so the entry that this link
#contains relates to the user's entry.  An example is given in
#the message which is: http://www.d.umn.edu/~rass0028/cgi-bin/post-test/browse.cgi?search=print&member=$num
#The portion of this link that must stay the same is /browse.cgi?search=print&member=$num
#The portion of the link that will change is the content before this section.
#It will change to where the maintainer has the site posted.  
print MAIL <<END_OF_MESSAGE;
To: $email
Reply-To: $REPLY
Subject: Registry of Latin American Researchers Update Approval
   

Your Update to the Registry of Latin American Researchers in Natural 
Language Processing and Computational Linguistics has been approved.

Please check your entry for accuracy at:
http://www.d.umn.edu/~tpederse/registry/browse.cgi?search=print&member=$num  
and update as necessary. 

Make sure to come back and visit the Registry. We hope you'll find
the information useful. We'd be grateful for any comments or suggestions
you might have.

Please tell your students, friends, colleagues, etc.  about the Registry.
They can browse or join simply by visiting:

http://www.d.umn.edu/~tpederse/registry/registry.cgi

Thank you for your participation.

END_OF_MESSAGE
#close sendmail
close MAIL or dienice("Couldn't close sendmail: $! \n");

}


#this function will reject an addition to the registry
sub rejectadd{

    #Open pendingadd.txt to get the entry to process
    open(FP,"../pendingadd/pendingadd.txt") or dienice("Couldn't open pendingadd.txt: $! \n");

    #lock the file in case of multiple users
    flock(FP,2);

    #move to the beginning of the file
    seek(FP,0,0);

    #store the entry in $data and the rest of the file in @rest
    $data = <FP>;
    @rest = <FP>;

    #close file
    close(FP);

    #mark the data with the rejection sign
    $data = "R:" . $data;

    #open pendingadd.txt with write privilege to change its contents
    open(FP,">../pendingadd/pendingadd.txt") or dienice("Couldn't open pendingadd.txt: $! \n");

    #lock the file in case of multiple users
    flock(FP,2);

    #print data to the pendingadd.txt, at the end
    if(@rest){
	print FP @rest;
    }
    print FP $data;

    #close file
    close (FP);

    #remove rejection marking so the original entry can be used
    $data =~ s/^R://;

    #open rejectedadd.txt to append the rejected entry to the log
    open(FP2,">>../rejectedadd/rejectedadd.txt") or dienice("Couldn't open rejectedadd.txt: $! \n");

    #lock the file in case of multiple users
    flock(FP2,2);

    #move to the end of the file
    seek(FP2,0,2);

    #print the data at the end of rejectedadd.txt
    print FP2 "$data";

    #close file
    close(FP2);

    #split the entry up into the seperate fields
    ($num,$status, $last_name, $first_name, $affiliation, $department, $address, $address2, $city, $state, $postal_code,$country, $homepage, $homepage2, $email, $email2, $research, $origin_country,$date,$location) = split(/\|/, $data);

    #save the name of the transaction file in $filename
    $filename = $num."-".$date.".txt";

    #move the transaction to the rejected directory
    if(!(rename "../pendingadd/$filename","../rejectedadd/$filename")){
    dienice("Couldn't change $filename: $! \n");}


    #get the cgi that was passed in
    my $cgi = shift;
    
    #set a parameter so the next cgi can use the email of this user
    $cgi->param(r_email=>"$email");

print $cgi->header();
#this print statement will print out the rejection email html page.
print qq'
<html>
  <head>
    <title>Rejection Message</title>
   
</head>

<body bgcolor="#ffffff">
  <h2 align = "center">Rejection Message</h2><br>
<p align = center><font color = ff0000>Caution: The addition has been rejected.  Using your browser\'s back button will non bring the entry back.  You must either send a message, or reject without sending from this point.</font></p>

<p align = center>Message will be sent to: $email</p><br>
<hr noshade>

<form method = post>
';
print $cgi->hidden(-name=> "r_email", -value=> $cgi->param('r_email')=> -override, 1);
print qq'
<table><tr><td>
Rejection email message:</td>

<td>    <textarea rows=10 cols=40 name="reject" wrap="physical"></textarea><br></td></tr></table>
      <input type="submit" name = "send_rejection" value="Send Message">
      <input type="submit" name = "nosend_rejection" value="Reject with no message">
</form></body></html>

';
}

#this function will reject an entry update
sub rejectupdate{

    #open pendingupdate to get the entry to process
    open(FP,"../pendingupdate/pendingupdate.txt") or dienice("Couldn't open pendingupdate.txt: $! \n");

    #lock the file in case of multiple users
    flock(FP,2);

    #move to the beginning of the file
    seek(FP,0,0);

    #store the file in $data1, $data2 and @rest
    $data1 = <FP>;
    $data2 = <FP>;
    @rest = <FP>;

    #close file
    close(FP);
    
    #mark with the rejection sign (R:)
    $data1 = "R:" . $data1;
    $data2 = "R:" . $data2;
    
    #open pendingupdate.txt with write permissions to change its contents
    open(FP,">../pendingupdate/pendingupdate.txt") or dienice("Couldn't open pendingupdate.txt: $! \n");

    #lock the file in case of multiple users
    flock(FP,2);

    #print the data to file, with the entry at the end
    if(@rest){
	print FP @rest;
    }
    print FP $data1;
    print FP $data2;

    #close file
    close (FP);

    #remove the rejection sign to use the entry in its original form
    $data1 =~ s/^R://;
    $data2 =~ s/^R://;

    #append the entry to the end of rejectedupdate.txt
    open(FP2,">>../rejectedupdate/rejectedupdate.txt") or dienice("Couldn't open rejectedupdate.txt: $! \n");

    #lock the file in case of multiple users
    flock(FP2,2);

    #move to the end of the file
    seek(FP2,0,2);

    #print the entry at the end of rejectedupdate.txt
    print FP2 $data1;
    print FP2 $data2;

    #close file
    close(FP2);

    #split the information into seperate fields
    ($num,$status, $last_name, $first_name, $affiliation, $department, $address, $address2, $city, $state, $postal_code,$country, $homepage, $homepage2, $email, $email2, $research, $origin_country,$date,$location) = split(/\|/, $data2);

    #save the transaction file's name into $filename
    $filename = $num."-".$date.".txt";

    #move transaction file to the approved directory
    if(!(rename "../pendingupdate/$filename","../rejectedupdate/$filename")){
	dienice("Couldn't change $filename: $! \n");}

    #get the cgi that was passed in
    my $cgi = shift;
    
    #save the email as a parameter for the next cgi
    $cgi->param(r_email=>"$email");

#this print statement will print out the rejection email html page.
print qq'
<html>
  <head>
    <title>Rejection Message</title>
   
</head>

<body bgcolor="#ffffff">
  <h2 align = "center">Rejection Message</h2><br>
<p align = center><font color = ff0000>Caution: The update has been rejected.  Using your browser\'s back button will non bring the entry back.  You must either send a message, or reject without sending from this point.</font></p>
<p align = center>Message will be sent to: $email</p><br>
<hr noshade>

<form method = post>
';
print $cgi->hidden(-name=> "r_email", -value=> $cgi->param('r_email')=> -override, 1);
print qq'
<table><tr><td>
Rejection email message:</td>

<td>    <textarea rows=10 cols=40 name="reject" wrap="physical"></textarea><br></td></tr></table>
      <input type="submit" name = "send_rejection" value="Send Message">
      <input type="submit" name = "nosendupdate_rejection" value="Reject with no message">
</form></body></html>

';

}

#This function will send the message to the email address of the submitter
#It is used to send rejection messages specified by the administrator
sub sendrejection{
    
    my $cgi = shift;

    $message = $cgi->param('reject');
    $sendto = $cgi->param('r_email');
    
    #clear paths
    $ENV{PATH} = "/bin:/usr/bin";
    delete @ENV{ qw( IFS CDPATH ENV BASH_ENV ) };

    #open the sendmail program
    open MAIL, "| /usr/lib/sendmail -t -i" or dienice("Couldn't open sendmail: $! \n");
    
#############################################################
#CHANGE THE CONTENT OF THE AUTOMATED EMAIL HERE
#THIS IS THE NON-AUTOMATED REJECTION MESSAGE
#THIS MESSAGE IS DIFFERENT, AND IS ENTERED BY THE ADMINISTRATOR
#  WHO REJECTS THE ENTRY/APPROVAL
#############################################################
    #send message #CHANGE THE ADDRESSES LATER 
    print MAIL <<END_OF_MESSAGE;
To: $sendto 
Reply-To: $REPLY
Subject: Registry of Latin American Researchers Rejection

$message
END_OF_MESSAGE
#close sendmail
close MAIL or dienice("Couldn't close sendmail: $! \n");

    #go back to the main page
    rootapproval($cgi);   
}


#This function will print out a web page with all the new additions to approve
sub viewadds{
    #get the cgi that was passed in
    my $cgi = shift;
    my $count = 0;

    #print $cgi->header();
    #this print statement will print out the beginning of the html page.
    print qq'
<html>
  <head>
    <title>View All Additions</title>
   
</head>

<body bgcolor="#ffffff">

  <h2 align = "center">View All Additions</h2>

<hr noshade>
    
';
    #open pendingadd.txt to get the entries
    open(INFP,"../pendingadd/pendingadd.txt") or dienice("Couldn't open pendingadd.txt: $! \n");

    #lock the file in case of multiple users
    flock(INFP,2);

    #move to the beginning of the file
    seek(INFP,0,0);

    #store the data in @data
    @data = <INFP>;

    #close file
    close(INFP);
    
    #if there are entries to process
    if(@data){
	#for each entry $i
	foreach $i (@data){
	    #remove newline
	    chomp($i);
	    
	    #split the information
	    ($num,$status, $last_name, $first_name, $affiliation, $department, $address, $address2, $city, $state, $postal_code,$country, $homepage, $homepage2, $email, $email2, $research, $origin_country,$date,$location) = split(/\|/, $i);

	    #if it has the R: or A: flag, it has already been processed
	    if($num =~ /^R:.+/ || $num =~ /^A:.+/ || $num eq ""){
		#if count is 0, there are not any entries to process
		if($count == 0){
		    print "<br><b>No additions to approve</b>";
		    print "<br><a href = approve.cgi>Back</a>";
		}
		#else there have been entries printed, so just exit the loop
		last;
	    }
	    #if the entry does not have a R: or A:, print, and go on
	    else{
		$count++;
		
		#remove underscores from the values for easier reading
		$country =~ s/_/ /g;
		$origin_country =~ s/_/ /g;
		
		print_visitor();
	    }
	    
	}
    }
    #print the results of how many entries need to be processed
    print "<br> There are $count additions to approve.<br>";
    print "<a href = approve.cgi>Back</a>";
}

#This function will print out a web page with all the new updates to approve
sub viewupdates{
    #get the cgi that was passed in
    my $cgi = shift;
    my $count = 0;
    
    #print $cgi->header();
    #this print statement will print out the beginning of the html page.
    print qq'
<html>
  <head>
    <title>View All Updates</title>
   
</head>

<body bgcolor="#ffffff">

  <h2 align = "center">View All Updates</h2>

<hr noshade>
    
';

    #open pendingupdate.txt to get the information entries
    open(INFP,"../pendingupdate/pendingupdate.txt") or dienice("Couldn't open pendingupdate.txt: $! \n");

    #lock the file in case of multiple users
    flock(INFP,2);

    #move to the beginning of the file
    seek(INFP,0,0);

    #store the data in @data
    @data = <INFP>;

    #close file
    close(INFP);
    
    #if there is data, continue each
    if(@data){
	while(@data){
	    #reset the %diff hash
	    %diff = ();

	    #save the original and updated entry in $i and $j respectivly
	    $i = shift(@data);
	    $j = shift(@data);
	    
	    #remove newlines
	    chomp($i);
	    chomp($j);

	    #split the original information
	    ($num,$status, $last_name, $first_name, $affiliation, $department, $address, $address2, $city, $state, $postal_code,$country, $homepage, $homepage2, $email, $email2, $research, $origin_country,$date,$location) = split(/\|/, $i);
	    #split the updated information
	    ($n,$st, $la, $fi, $af, $de, $ad, $ad2, $ci, $ste, $po,$co, $ho, $ho2, $em, $em2, $re, $or) = split(/\|/, $j);

	    #keep track of the different fields for easier printing
	    $diff{'status'}++ if($status ne $st);
	    $diff{'first_name'}++ if($first_name ne $fi);
	    $diff{'last_name'}++ if($last_name ne $la);
	    $diff{'affiliation'}++ if($affiliation ne $af);
	    $diff{'department'}++ if($department ne $de);
	    $diff{'address'}++ if($address ne $ad);
	    $diff{'address2'}++ if($address2 ne $ad2);
	    $diff{'city'}++ if($city ne $ci);
	    $diff{'state'}++ if($state ne $ste);
	    $diff{'postal_code'}++ if($postal_code ne $po);
	    $diff{'country'}++ if($country ne $co);
	    $diff{'homepage'}++ if($homepage ne $ho);
	    $diff{'homepage2'}++ if($homepage2 ne $ho2);
	    $diff{'email'}++ if($email ne $em);
	    $diff{'email2'}++ if($email2 ne $em2);
	    $diff{'research'}++ if($research ne $re);
	    $diff{'origin_country'}++ if($origin_country ne $or);

	    #check for valid entries without R: or A:
	    if($num =~ /^R:.+/ || $num =~ /^A:.+/ || $num eq ""){
		#if this is not a new entry, and count=0, print that there
		#are no entries to approve
		if($count == 0){
		    print "<br><b>No updates to approve</b>";
		    print "<br><br><a href = approve.cgi>Back</a>";
		}
		#exit loop because the rest of the entries have been processed
		last;
	    }
	    #if this entry has not been processed yet
	    else{
		$count++;
		#remove underscores from the values
		$country =~ s/_/ /g;
		$origin_country =~ s/_/ /g;
		
		#start an html table for printing the entries
		print "<table> <tr><td>Original Entry</td><td></td><td>New Entry</td></tr><tr><td>";

		#print the original information
		print_visitor();
		print "</td><td></td><td>";
		
		#split the updated information
		($num,$status, $last_name, $first_name, $affiliation, $department, $address, $address2, $city, $state, $postal_code,$country, $homepage, $homepage2, $email, $email2, $research, $origin_country,$date,$location) = split(/\|/, $j);
		
		#remove underscores from the values
		$country =~ s/_/ /g;
		$origin_country =~ s/_/ /g;
	    
		#print the updated entry, and end the html table
		print_visitor();
		print "</td></tr></table><br><br>";
	    }
	    
	}
    }
    #print the results of how many updates need to be approved
    print "<br> There are $count updates to approve.<br>";
    print "<a href = approve.cgi>Back</a><br><br>";

}

#function will print an error message
sub dienice{
    my($msg) = @_;
    print "<h2Error</h2>\n";
    print $msg;
    exit;
}



